c $Id: constr.f,v 1.3 1999/04/26 16:28:47 jose Exp $

      subroutine constr( cell, na, isa, amass, xa, stress, fa )
c *****************************************************************
c User-written routine to implement specific geometric constraints,
c by orthogonalizing the forces and stress to undesired changes.
c Arguments:
c real*8  cell(3,3)    : input lattice vectors (Bohr)
c integer na           : input number of atoms
c integer isa(na)      : input species indexes
c real*8  amass(na)    : input atomic masses
c real*8  xa(3,na)     : input atomic cartesian coordinates (Bohr)
c real*8  stress( 3,3) : input/output stress tensor (Ry/Bohr**3)
c real*8  fa(3,na)     : input/output atomic forces (Ry/Bohr)
c *****************************************************************
      implicit         none
      integer          na, isa(na)
      double precision amass(na), cell(3,3), fa(3,na),
     .                 stress(3,3), xa(3,na)

c Write here your problem-specific code.

C PARAMETERS...............
C natoms: number of atoms
C Nsym:   number of symmetries (rotations)
C Maxsym: maximum number of succesive symmetries to find an atom
      integer natoms,Nsym,Maxsym
      parameter (natoms = 60)
      parameter (Nsym = 31)
      parameter (Maxsym = 4)
C .............


      real*8 tol, dist,Ri(natoms,3),n(3),Ro(natoms,3),phi,pi,
     .       rota(natoms,3,3),roti(Maxsym,3,3),rot(3,3),
     .       Ro1(natoms,1),Ro2(natoms,3),Ro3(natoms,3),Ro4(natoms,4)
      real*8 f(3,na),Ang,eV
      integer deg,i,ialr,ichannel,irot,irot1,irot2,irot3,irot4,
     .        isym,isym1,isym2,isym3,isym4,
     .        ix,j,jx,kx,lx,ii,jj,mm,nn,iix,jjx,trans(natoms)
      logical found(natoms)
      pi=4.0*atan(1.0)
      Ang = 1.d0 / 0.529177d0
      eV  = 1.d0 / 13.60580d0

C Read atomic coordinates....
      do i = 1, natoms
        do ix=1,3
          Ri(i,ix) = xa(ix,i)
        enddo
      enddo
C ....

C**** Pruneda ****
      ialr = 1
      tol = 2.0d-2
C*****************

C Set up list of atoms found by symmetry (1=found, 0=not found)...
      do i=1,natoms
        found(i)=.false.
      enddo
      found(ialr)=.true.
      do ix=1,3
        do jx=1,3
          rota(ialr,ix,jx)=0.0
        enddo
        rota(ialr,ix,ix)=1.0
      enddo
	
C ...

C Generate atoms starting from atom number IALR ............

      do i=1,natoms
***TEST***
        write(6,*) 'atomo:  ',i
        if (found(i)) goto 1000

C One symmetry operation....
        do isym=1,Nsym
          call icosahedral(isym,n,deg)
C number of rotations: deg-1
          do irot=1,deg-1
            phi=(2.*pi/deg)*irot
            call rotate(natoms,Ri,n,phi,Ro,rot)

c check if atom ialr has been transformed in atom i
            dist = sqrt((Ro(ialr,1)-Ri(i,1))**2 +
     .                 (Ro(ialr,2)-Ri(i,2))**2 +
     .                 (Ro(ialr,3)-Ri(i,3))**2)
            if (dist .le. tol) then
              found(i) = .true.
              do ix=1,3
                do jx=1,3
                  rota(i,ix,jx)=rot(ix,jx)
                enddo
              enddo
              goto 1000
            endif
          enddo
        enddo

C Two symmetry operations....
        do isym1=1,Nsym
        call icosahedral(isym1,n,deg)
C number of rotations: deg-1
        do irot1=1,deg-1
        phi=(2.*pi/deg)*irot1
        call rotate(natoms,Ri,n,phi,Ro1,rot)
        do ix=1,3
        do jx=1,3
        roti(1,ix,jx)=rot(ix,jx)
        enddo
        enddo
        do isym2=1,Nsym
        call icosahedral(isym2,n,deg)
C number of rotations: deg-1
        do irot2=1,deg-1
        phi=(2.*pi/deg)*irot2
        call rotate(natoms,Ro1,n,phi,Ro,rot)
        do ix=1,3
        do jx=1,3
        roti(2,ix,jx)=rot(ix,jx)
        enddo
        enddo

c check if atom ialr has been transformed in atom i
            dist = sqrt((Ro(ialr,1)-Ri(i,1))**2 +
     .                 (Ro(ialr,2)-Ri(i,2))**2 +
     .                 (Ro(ialr,3)-Ri(i,3))**2)
            if (dist .le. tol) then
              found(i) = .true.
              do ix=1,3
                do jx=1,3
                  rota(i,ix,jx)=0.0
                  do kx=1,3
                    rota(i,ix,jx)=roti(1,ix,kx)*roti(2,kx,jx)
                  enddo
                enddo
              enddo
              goto 1000
            endif
        enddo
        enddo
        enddo
        enddo

C Three symmetry operations....
        do isym1=1,Nsym
        call icosahedraL(isym1,n,deg)
C number of rotations: deg-1
        do irot1=1,deg-1
        phi=(2.*pi/deg)*irot1
        call rotate(natoms,Ri,n,phi,Ro1,rot)
        do ix=1,3
        do jx=1,3
        roti(1,ix,jx)=rot(ix,jx)
        enddo
        enddo
        do isym2=1,Nsym
        call icosahedraL(isym2,n,deg)
C number of rotations: deg-1
        do irot2=1,deg-1
        phi=(2.*pi/deg)*irot2
        call rotate(natoms,Ro1,n,phi,Ro2,rot)
        do ix=1,3
        do jx=1,3
        roti(2,ix,jx)=rot(ix,jx)
        enddo
        enddo
        do isym3=1,Nsym
        call icosahedraL(isym3,n,deg)
C number of rotations: deg-1
        do irot3=1,deg-1
        phi=(2.*pi/deg)*irot3
        call rotate(natoms,Ro2,n,phi,Ro,rot)
        do ix=1,3
        do jx=1,3
        roti(3,ix,jx)=rot(ix,jx)
        enddo
        enddo

c check if atom ialr has been transformed in atom i
            dist = sqrt((Ro(ialr,1)-Ri(i,1))**2 +
     .                 (Ro(ialr,2)-Ri(i,2))**2 +
     .                 (Ro(ialr,3)-Ri(i,3))**2)
            if (dist .le. tol) then
              found(i) = .true.
              do ix=1,3
                do jx=1,3
                  rota(i,ix,jx)=0.0
                  do kx=1,3
                    do lx=1,3
                      rota(i,ix,jx)=
     .                  roti(1,ix,kx)*roti(2,kx,lx)*roti(3,lx,jx)
                    enddo
                  enddo
                enddo
              enddo
              goto 1000
            endif
        enddo
        enddo
        enddo
        enddo
        enddo
        enddo

1000    continue
      enddo


****TEST*******
c      do i= 1,natoms
c        write(6,*) 'atomo:',i,'found:',found(i)
c      enddo
****************
      do i=1,natoms
        if (.not. found(i)) then
          write(6,*) 'ERROR: Atom ',i,' was not found by symmetry'
          stop
        endif
      enddo

      do i=1,natoms
        found(i)=.false.
      enddo


C Generate forces by symmetry ..................................
****TEST****
      write(6,*) 'Generate forces by symmetry '

C Loop over atoms for which forces will be generated ............
      do i=1,natoms
        if(i.eq.ialr) goto 2000
*        if( found(i) .eqv. .false. ) goto 2000

C Find out how each atom transforms under rotation that brings
C atom ialr to atom i ...
        do ii=1,natoms
          do ix=1,3
            Ro(ii,ix)=0.0d0
            do jx=1,3
              Ro(ii,ix)=Ro(ii,ix)+rota(i,ix,jx)*Ri(ii,jx)
            enddo
          enddo
        enddo
C ...

C Check that atom ialr is transformed into atom i
        dist = sqrt( (Ri(i,1)-Ro(ialr,1))**2 +
     .               (Ri(i,2)-Ro(ialr,2))**2 +
     .               (Ri(i,3)-Ro(ialr,3))**2 )

        if (dist .gt. tol) then
	   write(6,*) 'ERROR: Atom ',i,' Dist=',dist
           stop
        endif


C Find out the correspondence between atoms before and after the rotation...
        do ii=1,natoms
          do jj=1,natoms
            dist = sqrt( (Ri(ii,1)-Ro(jj,1))**2 +
     .                   (Ri(ii,2)-Ro(jj,2))**2 +
     .                   (Ri(ii,3)-Ro(jj,3))**2 )
            if (dist .le. tol) then
              trans(ii)=jj
              found(ii)=.true.
	    endif
          enddo
        enddo

        do jj=1,natoms
          if (.not. found(jj)) then
            write(6,*) 'ERROR: Atom ',jj,' was not found by symmetry'
            stop
          endif
          found(jj)=.false.
        enddo
C ...

C Transform the forces...
	mm=trans(i)
	if (mm.ne.ialr) then
          write(6,*) 'ERROR: wrong symmetry!'
          stop
        endif
	do ix=1,3
          f(ix,i) = 0.0d0
          do lx=1,3
            f(ix,i) = f(ix,i) + rota(i,ix,lx) * fa(lx,mm)
          enddo
        enddo

2000    continue
      enddo
C .........

      do i = 2,natoms
        do ix = 1,3
          fa(ix,i) = f(ix,i)
        enddo
      enddo


      return
      end

      subroutine rotate(natoms,Ri,n,phi,Ro,rot)

C =============================================================
C  Rotates the input coordinates Ri around the axis defined by
C  unit vector n, by an angle phi.
C
C ------------------------- INPUT -----------------------------
C integer     natoms       Number of atoms
C real*8      Ri(natoms,3) Atomic coordinates
C real*8      n            Unit vector of the rotation axis
C real*8r     phi          The rotation angle (in radians)
C ------------------------- OUTPUT ----------------------------
C real*8      Ro(natoms,3) Rotated Atomic coordinates
C real*8      Rot(3,3)     Rotation matrix
C -------------------------------------------------------------
C Written by: P. Ordejon, June 1999
C =============================================================

      implicit none
      integer natoms
      real*8 Ri(natoms,3),n(3),Ro(natoms,3),phi,rot(3,3)

C Internal variables.........
      integer i,j,k,delta(3,3),eps(3,3,3)
C.........

C Define delta and epsilon ...........
      do i=1,3
        do j=1,3
	  if (i.eq.j) then
            delta(i,j)=1
	  else
            delta(i,j)=0
          endif
        enddo
      enddo

      do i=1,3
        do j=1,3
          do k=1,3
            eps(i,j,k)=0
          enddo
        enddo
      enddo

      eps(1,2,3)=1
      eps(2,3,1)=1
      eps(3,1,2)=1
      eps(2,1,3)=-1
      eps(1,3,2)=-1
      eps(3,2,1)=-1
C.............

C Rotation matrix...........
      do i=1,3
        do j=1,3
          rot(i,j) = cos(phi) * delta(i,j) +
     .               (1.-cos(phi)) * n(i) * n(j)
          do k=1,3
            rot(i,j) = rot(i,j) - sin(phi) * eps(i,j,k) * n(k)
          enddo
        enddo
      enddo
C.............

C Rotate coordinates............
      do i=1,natoms
        do j=1,3
          Ro(i,j)=0.0d0
          do k=1,3
            Ro(i,j) = Ro(i,j)+rot(j,k)*Ri(i,k)
          enddo
        enddo
      enddo
C...............

      return
      end



      subroutine icosahedral(isym,n,deg)

C -------------------------------------------------------------
C  Returns the symmetry axis unit vector and the order of the
C  rotation for the 31 rotation symmetries of C60
C
C  INPUT..... 
C     isym         Number of rotation (for C60: 31 rotations)
C     Ri(60,3)     Atomic coordinates
C  OUTPUT.... 
C     n            Unit vector of the rotation axis
C     deg          The rotation is deg -fold
C 
C Written by: J. M. Alonso Pruneda and P. Ordejon, June 1999
C -------------------------------------------------------------

      implicit none
      integer isym,deg
      real*8  n(3) 

C Internal variables
      real*8 R1,R2,R3,aux
      integer i,ix
      real*8 p1(3),p2(3),p3(3),p4(3),p5(3),p6(3),p7(3),p8(3),
     .         p9(3),p10(3),p11(3),p12(3)
      data p1  /  1.618033989d0 ,  1.0d0         ,  0.0d0         /
      data p2  / -1.618033989d0 ,  1.0d0         ,  0.0d0         /
      data p3  /  1.618033989d0 , -1.0d0         ,  0.0d0         /
      data p4  / -1.618033989d0 , -1.0d0         ,  0.0d0         /
      data p5  /  1.0d0         ,  0.0d0         ,  1.618033989d0 /
      data p6  / -1.0d0         ,  0.0d0         ,  1.618033989d0 /
      data p7  /  1.0d0         ,  0.0d0         , -1.618033989d0 /
      data p8  / -1.0d0         ,  0.0d0         , -1.618033989d0 /
      data p9  /  0.0d0         ,  1.618033989d0 ,  1.0d0         /
      data p10 /  0.0d0         , -1.618033989d0 ,  1.0d0         /
      data p11 /  0.0d0         ,  1.618033989d0 , -1.0d0         /
      data p12 /  0.0d0         , -1.618033989d0 , -1.0d0         /


      if (isym .gt. 31) then
         stop 'Wrong isym'
      endif

C...Ejes de simetria 5...

      if(isym.eq.1) then
        deg = 5
        aux = 0.d0
        do ix = 1,3
          n(ix) = p1(ix)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.2) then
        deg = 5
        aux = 0.d0
        do ix = 1,3
          n(ix) = p3(ix)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.3) then
        deg = 5
        aux = 0.d0
        do ix = 1,3
          n(ix) = p5(ix)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.4) then
        deg = 5
        aux = 0.d0
        do ix = 1,3
          n(ix) = p7(ix)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.5) then
        deg = 5
        aux = 0.d0
        do ix = 1,3
          n(ix) = p9(ix)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.6) then
        deg = 5
        aux = 0.d0
        do ix = 1,3
          n(ix) = p11(ix)
          aux = aux + n(ix)*n(ix)
        enddo

C...Symetrias de orden 3...

      else if(isym.eq.7) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p3(ix)
          R3 = p5(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.8) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p3(ix)
          R3 = p7(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.9) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p7(ix)
          R3 = p11(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.10) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p3(ix)
          R2 = p7(ix)
          R3 = p12(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.11) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p5(ix)
          R3 = p9(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.12) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p3(ix)
          R2 = p5(ix)
          R3 = p10(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.13) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p9(ix)
          R3 = p11(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.14) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p3(ix)
          R2 = p10(ix)
          R3 = p12(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.15) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p5(ix)
          R2 = p6(ix)
          R3 = p10(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.16) then
        deg = 3
        aux = 0.d0
        do ix = 1,3
          R1 = p5(ix)
          R2 = p6(ix)
          R3 = p9(ix)
          n(ix) = (R1+R2+R3)
          aux = aux + n(ix)*n(ix)
        enddo

C....Ejes de simetria 2....

      else if(isym.eq.17) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p3(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.18) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p2(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.19) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p5(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.20) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p7(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.21) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p5(ix)
          R2 = p3(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.22) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p7(ix)
          R2 = p3(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.23) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p5(ix)
          R2 = p6(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.24) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p9(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.25) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p1(ix)
          R2 = p11(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.26) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p10(ix)
          R2 = p3(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.27) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p12(ix)
          R2 = p3(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.28) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p5(ix)
          R2 = p9(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.29) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p5(ix)
          R2 = p10(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.30) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p7(ix)
          R2 = p11(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      else if(isym.eq.31) then
        deg = 2
        aux = 0.d0
        do ix = 1,3
          R1 = p7(ix)
          R2 = p12(ix)
          n(ix) = (R1+R2)
          aux = aux + n(ix)*n(ix)
        enddo
      endif
C ....................................      

C Normalizacion del vector unitario.

      do ix = 1,3
        n(ix) = n(ix)/sqrt(aux)
      enddo

      return


      end


